@@ -68,6 +68,11 @@ int setup_proxy(void)
 	bool exists_upper = false, exists_lower = false;
 	NihError *err;
 
+	/* When running in container, /sys/fs/cgroup will have been
+	   already mounted.  But it may be ro */
+	if (is_ro_mount(CGDIR))
+		turn_mount_rw(CGDIR);
+
 	/*
 	 * If /sys/fs/cgroup/cgmanager.lower exists,
 	 *    if /sys/fs/cgroup/cgmanager exists, then exit (proxy already running)
@@ -254,7 +259,7 @@ static bool complete_dbus_request(DBusMessage *message,
 	return true;
 }
 
-int get_pid_cgroup_main (void *parent, const char *controller,
+int get_pid_cgroup_main (void *parent, char *controller,
 		struct ucred p, struct ucred r, struct ucred v, char **output)
 {
 	DBusMessage *message;
@@ -304,7 +309,7 @@ out:
 	return ret;
 }
 
-int get_pid_cgroup_abs_main (void *parent, const char *controller,
+int get_pid_cgroup_abs_main (void *parent, char *controller,
 		struct ucred p, struct ucred r, struct ucred v, char **output)
 {
 	DBusMessage *message;
@@ -619,7 +624,7 @@ out:
 	return ret;
 }
 
-int get_value_main (void *parent, const char *controller, const char *cgroup,
+int get_value_main (void *parent, char *controller, const char *cgroup,
 		 const char *key, struct ucred p, struct ucred r, char **value)
 {
 	DBusMessage *message;
@@ -682,7 +687,7 @@ out:
 	return ret;
 }
 
-int set_value_main (const char *controller, const char *cgroup,
+int set_value_main (char *controller, const char *cgroup,
 		 const char *key, const char *value, struct ucred p,
 		 struct ucred r)
 {
@@ -805,7 +810,7 @@ out:
 	return ret;
 }
 
-int get_tasks_main (void *parent, const char *controller, const char *cgroup,
+int get_tasks_main (void *parent, char *controller, const char *cgroup,
 		    struct ucred p, struct ucred r, int32_t **pids)
 {
 	DBusMessage *message;
@@ -956,7 +961,7 @@ out:
 	return ret;
 }
 
-int list_children_main (void *parent, const char *controller, const char *cgroup,
+int list_children_main (void *parent, char *controller, const char *cgroup,
 		    struct ucred p, struct ucred r, char ***output)
 {
 	DBusMessage *message;
@@ -1244,7 +1249,7 @@ static char *find_eol(char *s)
 	return s;
 }
 
-int list_keys_main (void *parent, const char *controller, const char *cgroup,
+int list_keys_main (void *parent, char *controller, const char *cgroup,
 		    struct ucred p, struct ucred r,
 		    struct keys_return_type ***output)
 {
@@ -1454,10 +1459,5 @@ main (int argc, char *argv[])
 
 	ret = nih_main_loop ();
 
-	/* Destroy any PID file we may have created */
-	if (daemonise) {
-		nih_main_unlink_pidfile();
-	}
-
 	return ret;
 }
