@@ -19,7 +19,6 @@
 
 #include <frontend.h>
 #include <sys/resource.h>
-#include <sys/statvfs.h>
 #include <sys/vfs.h>
 #include <linux/fs.h>
 
@@ -32,11 +31,17 @@
 static int maxdepth = 16;
 
 /* GetPidCgroup */
-int get_pid_cgroup_main(void *parent, const char *controller,struct ucred p,
+int get_pid_cgroup_main(void *parent, char *controller, struct ucred p,
 			 struct ucred r, struct ucred v, char **output)
 {
 	char rcgpath[MAXPATHLEN], vcgpath[MAXPATHLEN];
 
+	if (!(prune_verify_comounts(controller))) {
+		nih_error("%s: Multiple controllers given: %s",
+				__func__, controller);
+		return -1;
+	}
+
 	// Get r's current cgroup in rcgpath
 	if (!compute_pid_cgroup(r.pid, controller, "", rcgpath, NULL)) {
 		nih_error("%s: Could not determine the requestor's cgroup for %s",
@@ -67,13 +72,19 @@ int get_pid_cgroup_main(void *parent, const char *controller,struct ucred p,
 }
 
 /* GetPidCgroupAbs */
-int get_pid_cgroup_abs_main(void *parent, const char *controller,struct ucred p,
+int get_pid_cgroup_abs_main(void *parent, char *controller,struct ucred p,
 			 struct ucred r, struct ucred v, char **output)
 {
 	char rcgpath[MAXPATHLEN], vcgpath[MAXPATHLEN];
 
+	if (!(prune_verify_comounts(controller))) {
+		nih_error("%s: Multiple controllers given: %s",
+				__func__, controller);
+		return -1;
+	}
+
 	// Get p's current cgroup in rcgpath
-	if (!compute_pid_cgroup(p.pid, controller, "", rcgpath, NULL)) {
+	if (!compute_proxy_cgroup(p.pid, controller, "", rcgpath, NULL)) {
 		nih_error("%s: Could not determine the requestor's cgroup for %s",
 				__func__, controller);
 		return -1;
@@ -120,13 +131,18 @@ int per_ctrl_move_pid_main(const char *controller, const char *cgroup, struct uc
 		struct ucred r, struct ucred v, bool escape)
 {
 	char rcgpath[MAXPATHLEN], path[MAXPATHLEN];
+	bool unified = false;
 	FILE *f;
 	pid_t query = r.pid;
+	size_t maxlen;
+
+	if (is_unified_controller(controller))
+		unified = true;
 
 	// Get r's current cgroup in rcgpath
 	if (escape)
 		query = p.pid;
-	if (!compute_pid_cgroup(query, controller, "", rcgpath, NULL)) {
+	if (!compute_proxy_cgroup(query, controller, "", rcgpath, NULL)) {
 		nih_error("%s: Could not determine the requestor's cgroup for %s",
                 __func__, controller);
 		return -1;
@@ -138,8 +154,14 @@ int per_ctrl_move_pid_main(const char *controller, const char *cgroup, struct uc
 		return -1;
 	}
 
-	/* rcgpath + / + cgroup + /tasks + \0 */
-	if (strlen(rcgpath) + strlen(cgroup) > MAXPATHLEN - 8) {
+	if (unified) {
+		/* rcgpath + / + cgroup + "/.cgm_leaf/cgroup.procs" + \0 */
+		maxlen = strlen(rcgpath) + strlen(cgroup) + strlen(U_LEAF "/cgroup.procs") + 2;
+	} else {
+		/* rcgpath + / + cgroup + /tasks + \0 */
+		maxlen = strlen(rcgpath) + strlen(cgroup) + 8;
+	}
+	if (maxlen > MAXPATHLEN) {
 		nih_error("%s: Path name too long", __func__);
 		return -1;
 	}
@@ -156,8 +178,17 @@ int per_ctrl_move_pid_main(const char *controller, const char *cgroup, struct uc
 			r.pid, r.uid, r.gid, path);
 		return -1;
 	}
+
 	// is r allowed to write to tasks file?
-	strncat(path, "/tasks", MAXPATHLEN-1);
+	if (unified) {
+		if (!ensure_leafdir(controller, path))
+			return -1;
+		strcat(path, U_LEAF);
+		strcat(path, "/cgroup.procs");
+	} else {
+		strncat(path, "/tasks", MAXPATHLEN-1);
+	}
+
 	if (!may_access(r.pid, r.uid, r.gid, path, O_WRONLY)) {
 		nih_error("%s: pid %d (uid %u gid %u) may not write to %s", __func__,
 			r.pid, r.uid, r.gid, path);
@@ -312,6 +343,12 @@ int do_create_main(const char *controller, const char *cgroup, struct ucred p,
 			nih_error("%s: failed to create %s", __func__, path);
 			return -2;
 		}
+		if (!unified_copy_controllers(controller, path)) {
+			nih_error("%s: Failed to set cg controllers on %s", __func__,
+					path);
+			rmdir(path);
+			return -1;
+		}
 		if (!chown_cgroup_path(path, r.uid, r.gid, true)) {
 			nih_error("%s: Failed to change ownership on %s to %u:%u", __func__,
 				path, r.uid, r.gid);
@@ -418,6 +455,14 @@ int do_chown_main(const char *controller, const char *cgroup, struct ucred p,
 			path, v.uid, v.gid);
 		return -2;
 	}
+	if (is_unified_controller(controller)) {
+		NIH_MUST( nih_strcat(&path, NULL, U_LEAF) );
+		if (dir_exists(path) && !chown_cgroup_path(path, v.uid, v.gid, false)) {
+			nih_warn("%s: Failed to chown leaf directory for %s to %u:%u",
+				__func__, path, v.uid, v.gid);
+			return -2;
+		}
+	}
 
 	return 0;
 }
@@ -554,11 +599,17 @@ next:
 	return 0;
 }
 
-int get_value_main(void *parent, const char *controller, const char *cgroup,
+int get_value_main(void *parent, char *controller, const char *cgroup,
 		const char *key, struct ucred p, struct ucred r, char **value)
 {
 	char path[MAXPATHLEN];
 
+	if (!(prune_verify_comounts(controller))) {
+		nih_error("%s: Multiple controllers given: %s",
+				__func__, controller);
+		return -1;
+	}
+
 	if (!sane_cgroup(cgroup)) {
 		nih_error("%s: unsafe cgroup", __func__);
 		return -1;
@@ -572,12 +623,12 @@ int get_value_main(void *parent, const char *controller, const char *cgroup,
 
 	/* Check access rights to the cgroup directory */
 	if (!may_access(r.pid, r.uid, r.gid, path, O_RDONLY)) {
-		nih_error("%s: Pid %d may not access %s\n", __func__, r.pid, path);
+		nih_debug("%s: Pid %d may not access %s\n", __func__, r.pid, path);
 		return -1;
 	}
 
-	if (!path_is_under_taskcg(p.pid, controller, path)) {
-		nih_error("%s: target cgroup is not below r (%d)'s", __func__,
+	if (!path_is_under_proxycg(p.pid, controller, path)) {
+		nih_debug("%s: target cgroup is not below r (%d)'s", __func__,
 			r.pid);
 		return -1;
 	}
@@ -593,7 +644,7 @@ int get_value_main(void *parent, const char *controller, const char *cgroup,
 
 	/* Check access rights to the file itself */
 	if (!may_access(r.pid, r.uid, r.gid, path, O_RDONLY)) {
-		nih_error("%s: Pid %d may not access %s\n", __func__, r.pid, path);
+		nih_debug("%s: Pid %d may not access %s\n", __func__, r.pid, path);
 		return -1;
 	}
 
@@ -608,13 +659,19 @@ int get_value_main(void *parent, const char *controller, const char *cgroup,
 	return 0;
 }
 
-int set_value_main(const char *controller, const char *cgroup,
+int set_value_main(char *controller, const char *cgroup,
 		const char *key, const char *value, struct ucred p,
 		struct ucred r)
 
 {
 	char path[MAXPATHLEN];
 
+	if (!(prune_verify_comounts(controller))) {
+		nih_error("%s: Multiple controllers given: %s",
+				__func__, controller);
+		return -1;
+	}
+
 	if (!sane_cgroup(cgroup)) {
 		nih_error("%s: unsafe cgroup", __func__);
 		return -1;
@@ -626,15 +683,15 @@ int set_value_main(const char *controller, const char *cgroup,
 		return -1;
 	}
 
-	if (!path_is_under_taskcg(p.pid, controller, path)) {
-		nih_error("%s: target cgroup is not below r (%d)'s", __func__,
+	if (!path_is_under_proxycg(p.pid, controller, path)) {
+		nih_debug("%s: target cgroup is not below r (%d)'s", __func__,
 			r.pid);
 		return -1;
 	}
 
 	/* Check access rights to the cgroup directory */
 	if (!may_access(r.pid, r.uid, r.gid, path, O_RDONLY)) {
-		nih_error("%s: Pid %d may not access %s\n", __func__, r.pid, path);
+		nih_debug("%s: Pid %d may not access %s\n", __func__, r.pid, path);
 		return -1;
 	}
 
@@ -649,12 +706,12 @@ int set_value_main(const char *controller, const char *cgroup,
 
 	/* Check access rights to the file itself */
 	if (!may_access(r.pid, r.uid, r.gid, path, O_WRONLY)) {
-		nih_error("%s: Pid %d may not access %s\n", __func__, r.pid, path);
+		nih_debug("%s: Pid %d may not access %s\n", __func__, r.pid, path);
 		return -1;
 	}
 
 	/* read and return the value */
-	if (!set_value(path, value)) {
+	if (!set_value(controller, path, value)) {
 		nih_error("%s: Failed to set value %s to %s", __func__, path, value);
 		return -1;
 	}
@@ -777,6 +834,14 @@ int do_remove_main(const char *controller, const char *cgroup, struct ucred p,
 		return -2;
 	}
 
+	if (is_unified_controller(controller)) {
+		nih_local char *fpath = NULL;
+		fpath = NIH_MUST( nih_sprintf(NULL, "%s%s", working, U_LEAF) );
+		if (rmdir(fpath) < 0) {
+			nih_error("%s: Failed to remove %s: %s", __func__, fpath, strerror(errno));
+			return errno == EPERM ? -2 : -1;
+		}
+	}
 	if (!recursive) {
 		if (rmdir(working) < 0) {
 			nih_error("%s: Failed to remove %s: %s", __func__, working, strerror(errno));
@@ -831,7 +896,7 @@ next:
 	return 0;
 }
 
-int get_tasks_main(void *parent, const char *controller, const char *cgroup,
+int get_tasks_main(void *parent, char *controller, const char *cgroup,
 			struct ucred p, struct ucred r, int32_t **pids)
 {
 	char path[MAXPATHLEN];
@@ -843,21 +908,27 @@ int get_tasks_main(void *parent, const char *controller, const char *cgroup,
 		return -1;
 	}
 
+	if (!(prune_verify_comounts(controller))) {
+		nih_error("%s: Multiple controllers given: %s",
+				__func__, controller);
+		return -1;
+	}
+
 	if (!compute_pid_cgroup(r.pid, controller, cgroup, path, NULL)) {
 		nih_error("%s: Could not determine the requested cgroup (%s:%s)",
                 __func__, controller, cgroup);
 		return -1;
 	}
 
-	if (!path_is_under_taskcg(p.pid, controller, path)) {
-		nih_error("%s: target cgroup is not below r (%d)'s", __func__,
+	if (!path_is_under_proxycg(p.pid, controller, path)) {
+		nih_debug("%s: target cgroup is not below r (%d)'s", __func__,
 			r.pid);
 		return -1;
 	}
 
 	/* Check access rights to the cgroup directory */
 	if (!may_access(r.pid, r.uid, r.gid, path, O_RDONLY)) {
-		nih_error("%s: Pid %d may not access %s\n", __func__, r.pid, path);
+		nih_debug("%s: Pid %d may not access %s\n", __func__, r.pid, path);
 		return -1;
 	}
 
@@ -938,15 +1009,15 @@ int collect_tasks(void *parent, const char *controller, const char *cgroup,
 		return -2;
 	}
 
-	if (!path_is_under_taskcg(p.pid, controller, path)) {
-		nih_error("%s: target cgroup is not below r (%d)'s", __func__,
+	if (!path_is_under_proxycg(p.pid, controller, path)) {
+		nih_debug("%s: target cgroup is not below r (%d)'s", __func__,
 			r.pid);
 		return -1;
 	}
 
 	/* Check access rights to the cgroup directory */
 	if (!may_access(r.pid, r.uid, r.gid, path, O_RDONLY)) {
-		nih_error("%s: Pid %d may not access %s\n", __func__, r.pid, path);
+		nih_debug("%s: Pid %d may not access %s\n", __func__, r.pid, path);
 		return -2;
 	}
 
@@ -1005,7 +1076,7 @@ next:
 	return nrpids;
 }
 
-int list_children_main(void *parent, const char *controller, const char *cgroup,
+int list_children_main(void *parent, char *controller, const char *cgroup,
 			struct ucred p, struct ucred r, char ***output)
 {
 	char path[MAXPATHLEN];
@@ -1016,21 +1087,27 @@ int list_children_main(void *parent, const char *controller, const char *cgroup,
 		return -1;
 	}
 
+	if (!(prune_verify_comounts(controller))) {
+		nih_error("%s: Multiple controllers given: %s",
+				__func__, controller);
+		return -1;
+	}
+
 	if (!compute_pid_cgroup(r.pid, controller, cgroup, path, NULL)) {
 		nih_error("%s: Could not determine the requested cgroup (%s:%s)",
                 __func__, controller, cgroup);
 		return -1;
 	}
 
-	if (!path_is_under_taskcg(p.pid, controller, path)) {
-		nih_error("%s: target cgroup is not below r (%d)'s", __func__,
+	if (!path_is_under_proxycg(p.pid, controller, path)) {
+		nih_debug("%s: target cgroup is not below r (%d)'s", __func__,
 			r.pid);
 		return -1;
 	}
 
 	/* Check access rights to the cgroup directory */
 	if (!may_access(r.pid, r.uid, r.gid, path, O_RDONLY)) {
-		nih_error("%s: Pid %d may not access %s\n", __func__, r.pid, path);
+		nih_debug("%s: Pid %d may not access %s\n", __func__, r.pid, path);
 		return -1;
 	}
 
@@ -1259,7 +1336,7 @@ int list_controllers_main(void *parent, char ***output)
 	return 0;
 }
 
-int list_keys_main(void *parent, const char *controller, const char *cgroup,
+int list_keys_main(void *parent, char *controller, const char *cgroup,
 			struct ucred p, struct ucred r,
 			struct keys_return_type ***output)
 {
@@ -1272,6 +1349,12 @@ int list_keys_main(void *parent, const char *controller, const char *cgroup,
 		return -1;
 	}
 
+	if (!(prune_verify_comounts(controller))) {
+		nih_error("%s: Multiple controllers given: %s",
+				__func__, controller);
+		return -1;
+	}
+
 	if (!compute_pid_cgroup(r.pid, controller, cgroup, path, NULL)) {
 		nih_error("%s: Could not determine the requested cgroup (%s:%s)",
                 __func__, controller, cgroup);
@@ -1280,7 +1363,7 @@ int list_keys_main(void *parent, const char *controller, const char *cgroup,
 
 	/* Check access rights to the cgroup directory */
 	if (!may_access(r.pid, r.uid, r.gid, path, O_RDONLY)) {
-		nih_error("%s: Pid %d may not access %s\n", __func__, r.pid, path);
+		nih_debug("%s: Pid %d may not access %s\n", __func__, r.pid, path);
 		return -1;
 	}
 
@@ -1378,27 +1461,6 @@ static void mount_tmpfs_at(const char *path)
 	nih_debug("Mounted tmpfs onto %s", path);
 }
 
-static bool is_ro_mount(const char *path)
-{
-	struct statvfs sb;
-
-	if (statvfs(path, &sb) < 0) {
-		nih_error("statvfs failed on %s: %m", path);
-		exit(1);
-	}
-
-	return sb.f_flag & MS_RDONLY;
-}
-
-static void turn_mount_rw(const char *path)
-{
-	if (mount("", path, "cgroup", MS_REMOUNT, NULL) < 0) {
-		nih_error("Failed to turn %s mount readonly: %m", path);
-		exit(1);
-	}
-	nih_debug("Turned %s from ro->rw", path);
-}
-
 /*
  * We may decide to make the socket path customizable.  For now
  * just assume it is in /sys/fs/cgroup/ which has some special
@@ -1520,10 +1582,5 @@ main (int argc, char *argv[])
 
 	ret = nih_main_loop ();
 
-	/* Destroy any PID file we may have created */
-	if (daemonise) {
-		nih_main_unlink_pidfile();
-	}
-
 	return ret;
 }
